/*@lineinfo:filename=CollectionExample3*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program CollectionExample3.sqlj illustrates how to retrieve,
   add and modify a nested table containing VARCHAR2 strings.
   This program uses the customers5 table that contains a column named
   addresses, which is a nested table that provides storage space any
   number of VARCHAR2 strings.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class CollectionExample3 {

  /*@lineinfo:generated-code*//*@lineinfo:14^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
    addressesNdx = findColumn("addresses");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
  public TAddress4 addresses() 
    throws java.sql.SQLException 
  {
    return ((TAddress4)(resultSet.getObject(addressesNdx, TAddress4.class)));
  }
  private int addressesNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "object_user",
        "object_password"
      );

      displayCustomers();

      // create an array of strings to store the addresses
      String [] address_strings = new String[3];
      address_strings[0] = "1 Any Street, Any Town, CA, 12345";
      address_strings[1] = "2 Liberty Street, Uptown, CA, 54321";
      address_strings[2] = "17 Main Street, New Town, CA, 51234";

      // create a host object named addresses and populate the object
      // by passing the address_strings array to the constructor
      TAddress4 addresses = new TAddress4(address_strings);

      // add a new customer
      addCustomer(3, "Jerry", "Fieldtop", addresses);

      // add a new string to the nested table
      /*@lineinfo:generated-code*//*@lineinfo:44^7*/

//  ************************************************************
//  #sql { INSERT INTO TABLE
//            (SELECT addresses FROM customers5 WHERE id = 3)
//          VALUES
//            ('3 A Street, Oldtown, CA, 34512')
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample3_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:49^7*/
      System.out.println("Successfully added new string to nested table.");

      // modify the first string in the nested table
      /*@lineinfo:generated-code*//*@lineinfo:53^7*/

//  ************************************************************
//  #sql { UPDATE TABLE
//            (SELECT addresses FROM customers5 WHERE id = 3) a
//          SET
//            VALUE (a) = '1 New Street, Newtown, CA, 34512'
//          WHERE
//            VALUE (a) = '1 Any Street, Any Town, CA, 12345'
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample3_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:60^7*/
      System.out.println("Successfully modified first string in nested " +
        "table.");

      // remove the second string from the nested table
      /*@lineinfo:generated-code*//*@lineinfo:65^7*/

//  ************************************************************
//  #sql { DELETE FROM TABLE
//            (SELECT addresses FROM customers5 WHERE id = 3) a
//          WHERE
//            VALUE (a) = '2 Liberty Street, Uptown, CA, 54321'
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample3_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:70^7*/
      System.out.println("Successfully deleted second string from " +
        "nested table.");

      displayCustomers();

      // rollback changes and disconnect from database
      /*@lineinfo:generated-code*//*@lineinfo:77^7*/

//  ************************************************************
//  #sql { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample3_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:77^23*/
      Oracle.close();

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void displayCustomers()
  throws SQLException {

    CustomerIteratorClass customer_iterator;

    /*@lineinfo:generated-code*//*@lineinfo:95^5*/

//  ************************************************************
//  #sql customer_iterator = { SELECT
//          id, first_name, last_name, addresses
//        FROM
//          customers5
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample3_SJProfileKeys.getKey(0), 4);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:102^5*/

    System.out.println("List of all customers ...");

    while (customer_iterator.next()) {

      System.out.println("Customer:");
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());

      // get the total number of elements using the length() method
      // through addresses()
      int address_count = customer_iterator.addresses().length();

      // retrieve the strings stored in the host object using the
      // getArray() method through addresses()
      String [] address_array = customer_iterator.addresses().getArray();

      // display the address strings
      System.out.println("Total number of addresses = " + address_count);
      for (int counter = 0; counter < address_count; counter++) {
        System.out.println("Address " + counter + ":");
        System.out.println("Entire address = " + address_array[counter]);
      }

      // retrieve a single string stored in the host object using the
      // getElement() method through addresses()
      String address = customer_iterator.addresses().getElement(0);
      System.out.println("First address: " + address);

    } // end of while loop

    customer_iterator.close();

  } // end of displayCustomers()


  private static void addCustomer (

    int id,
    String first_name,
    String last_name,
    TAddress4 addresses

  ) throws SQLException {

    /*@lineinfo:generated-code*//*@lineinfo:149^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          customers5 (
//            id, first_name, last_name, addresses
//          ) VALUES (
//            :id, :first_name, :last_name, :addresses
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  String __sJT_2 = first_name;
  String __sJT_3 = last_name;
  TAddress4 __sJT_4 = addresses;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, CollectionExample3_SJProfileKeys.getKey(0), 5);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setString(3, __sJT_3);
      __sJT_stmt.setObject(4, __sJT_4);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:156^5*/

  } // end of addCustomer()

}/*@lineinfo:generated-code*/class CollectionExample3_SJProfileKeys 
{
  private static CollectionExample3_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new CollectionExample3_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private CollectionExample3_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "CollectionExample3_SJProfile0");
  }
}
