/*@lineinfo:filename=FundamentalExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program FundamentalExample2 illustrates the use of a named iterator
   to display the id, name, description and price columns from the products
   table.
*/

// import required packages
import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class FundamentalExample2 {

  // step 1: declare the iterator class (remember that the iterator column names
  // must match the database column names in the SELECT statement, but the order
  // of the iterator columns doesn't have to match the order of the columns in the
  // SELECT statement; in the example below, the price iterator column occupies
  // position 2 but in the SELECT statement listed later in the program
  // the price database column is last)
  /*@lineinfo:generated-code*//*@lineinfo:19^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class ProductIteratorNamedClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public ProductIteratorNamedClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    priceNdx = findColumn("price");
    nameNdx = findColumn("name");
    descriptionNdx = findColumn("description");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public double price() 
    throws java.sql.SQLException 
  {
    return resultSet.getDoubleNoNull(priceNdx);
  }
  private int priceNdx;
  public String name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(nameNdx);
  }
  private int nameNdx;
  public String description() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(descriptionNdx);
  }
  private int descriptionNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:24^3*/

  public static void main(String [] args) throws SQLException {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // step 2: declare an iterator object from the iterator class
      ProductIteratorNamedClass product_iterator;

      // step 3: populate the iterator object using a SELECT statement
      /*@lineinfo:generated-code*//*@lineinfo:40^7*/

//  ************************************************************
//  #sql product_iterator = { SELECT
//            id, name, description, price
//          FROM
//            products
//          WHERE
//            id <= 5
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, FundamentalExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      product_iterator = new ProductIteratorNamedClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:47^7*/

      System.out.println("Product details:");

      // step 4: read the rows from the iterator object
      while (product_iterator.next()) {

        // print the contents of the iterator row variables
        System.out.println("id = " + product_iterator.id());
        System.out.println("name = " + product_iterator.name());
        System.out.println("description = " +
          product_iterator.description());
        System.out.println("price = " + product_iterator.price());

      } // end of while loop

      // step 5: close the iterator object
      product_iterator.close();

      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()

}/*@lineinfo:generated-code*/class FundamentalExample2_SJProfileKeys 
{
  private static FundamentalExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new FundamentalExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private FundamentalExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "FundamentalExample2_SJProfile0");
  }
}
