/*@lineinfo:filename=FundamentalExample4*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program FundamentalExample4 illustrates the use of
   using nested cursors with named iterators.
*/

// import required packages
import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class FundamentalExample4 {

  // declare the named iterator classes
  /*@lineinfo:generated-code*//*@lineinfo:13^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

public static class ProductDetailNamedIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public ProductDetailNamedIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    nameNdx = findColumn("name");
    priceNdx = findColumn("price");
  }
  public String name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(nameNdx);
  }
  private int nameNdx;
  public double price() 
    throws java.sql.SQLException 
  {
    return resultSet.getDoubleNoNull(priceNdx);
  }
  private int priceNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:16^3*/
  /*@lineinfo:generated-code*//*@lineinfo:17^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class ProductMasterNamedIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public ProductMasterNamedIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    nameNdx = findColumn("name");
    product_detailNdx = findColumn("product_detail");
  }
  public String name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(nameNdx);
  }
  private int nameNdx;
  public ProductDetailNamedIteratorClass product_detail() 
    throws java.sql.SQLException 
  {
    return ((ProductDetailNamedIteratorClass)(resultSet.getObject(product_detailNdx, ProductDetailNamedIteratorClass.class)));
  }
  private int product_detailNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:20^3*/

  public static void main(String [] args) throws SQLException {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // declare an iterator object to store the master rows
      ProductMasterNamedIteratorClass product_master_iterator;

      // populate the master iterator object using a SELECT statement
      // containing the nested cursor
      /*@lineinfo:generated-code*//*@lineinfo:37^7*/

//  ************************************************************
//  #sql product_master_iterator = { SELECT
//            name, CURSOR (
//              SELECT
//                name, price
//              FROM
//                products
//              WHERE
//                products.type_id = product_types.id
//            ) AS product_detail
//          FROM
//            product_types
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, FundamentalExample4_SJProfileKeys.getKey(0), 0);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      product_master_iterator = new ProductMasterNamedIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:49^7*/

      // read the master rows from the master iterator object
      while (product_master_iterator.next()) {

        // display the product type
        System.out.println("Product type = " + product_master_iterator.name());

        // instantiate a detail iterator object to read the product detail rows
        ProductDetailNamedIteratorClass product_detail_iterator;

        // set the detail iterator object to the detail rows in the
        // master iterator object
        product_detail_iterator = product_master_iterator.product_detail();

        // read the detail rows from the detail iterator object
        while (product_detail_iterator.next()) {

          // display the product name and price
          System.out.println("  Product name = " + product_detail_iterator.name() +
            ", price = " + product_detail_iterator.price());

        } // end of inner while loop

        // close the detail iterator
        product_detail_iterator.close();

      } // end of outer while loop

      // close the master iterator object
      product_master_iterator.close();

      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()

}/*@lineinfo:generated-code*/class FundamentalExample4_SJProfileKeys 
{
  private static FundamentalExample4_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new FundamentalExample4_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private FundamentalExample4_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "FundamentalExample4_SJProfile0");
  }
}
