/*@lineinfo:filename=JdbcSqljExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
  The program JdbcSqljExample1.sqlj illustrates the use of sharing
  database connections and transactions between SQLJ and JDBC.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;
import sqlj.runtime.ConnectionContext;

public class JdbcSqljExample1 {

  public static void main(String [] args) {

    try {

      // create a SQLJ connection context
      System.out.println("Connecting to the database using SQLJ.");
      DefaultContext sqlj_conn_context = Oracle.getConnection(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // assign the underlying JDBC connection for the default SQLJ connection
      // context to a new JDBC connection
      System.out.println("Sharing SQLJ connection with JDBC.");
      Connection jdbc_conn = sqlj_conn_context.getConnection();

      // display customer using SQLJ connection context
      displayCustomer(sqlj_conn_context, 1);

      // display customer using JDBC connection
      displayCustomer(jdbc_conn, 1);

      // update customer name using SQLJ connection context
      System.out.println("Updating customer 1 name to Fred Heyson using " +
        "SQLJ connection.");
      /*@lineinfo:generated-code*//*@lineinfo:39^7*/

//  ************************************************************
//  #sql [sqlj_conn_context] { UPDATE
//            customers
//          SET
//            first_name = 'Fred',
//            last_name  = 'Heyson'
//          WHERE
//            id = 1
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj_conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, JdbcSqljExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:47^7*/

      // display customer using SQLJ connection context
      displayCustomer(sqlj_conn_context, 1);

      // display customer using JDBC connection, the update is
      // visible because it shares the same transaction
      // as the SQLJ connection context
      displayCustomer(jdbc_conn, 1);

      // perform an explicit ROLLBACK using SQLJ connection context
      System.out.println("Performing rollback using SQLJ connection.");
      /*@lineinfo:generated-code*//*@lineinfo:59^7*/

//  ************************************************************
//  #sql [sqlj_conn_context] { ROLLBACK  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj_conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, JdbcSqljExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:59^43*/

      // close the SQLJ connection context, but keep the underlying JDBC
      // connection open
      System.out.println("Closing SQLJ connection, but keeping JDBC " +
        "connection open.");
      sqlj_conn_context.close(ConnectionContext.KEEP_CONNECTION);

      // display customer again, this shows that the JDBC connection is
      // still open
      displayCustomer(jdbc_conn, 1);

      // close the JDBC connection
      jdbc_conn.close();

      // load the Oracle driver, this enables the JDBC driver manager
      // to communicate with the Oracle database
      DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());

      // create a new JDBC connection
      System.out.println("Connecting to the database using JDBC.");
      Connection jdbc_conn2 = DriverManager.getConnection(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // create a new SQLJ connection context and pass the JDBC connection
      // into the constructor of DefaultContext, this shares the database
      // connection with SQLJ
      System.out.println("Sharing JDBC connection with SQLJ.");
      DefaultContext sqlj_conn_context2 = new DefaultContext(jdbc_conn2);

      displayCustomer(sqlj_conn_context2, 1);

      // close the SQLJ connection context, which also closes the
      // JDBC connection
      sqlj_conn_context2.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void displayCustomer(
    DefaultContext conn_context,
    int id
  ) throws SQLException {

    String first_name;
    String last_name;

    /*@lineinfo:generated-code*//*@lineinfo:116^5*/

//  ************************************************************
//  #sql [conn_context] { SELECT
//          first_name, last_name
//        
//        FROM
//          customers
//        WHERE
//          id = :id
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = id;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, JdbcSqljExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 2);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    first_name = __sJT_rtRs.getString(1);
    last_name = __sJT_rtRs.getString(2);
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:125^5*/

    System.out.println("Customer " + id +
      " details using SQLJ connection:");
    System.out.println("first_name = " + first_name);
    System.out.println("last_name = " + last_name);

  } // end of displayCustomer()


  private static void displayCustomer(
    Connection jdbc_conn,
    int id
  ) throws SQLException {

    String sql_statement = "SELECT first_name, last_name FROM " +
      "customers WHERE id = " + id;

    // create a prepared statement
    PreparedStatement prepared_statement =
      jdbc_conn.prepareStatement(sql_statement);

    // create and populate a result set with the retrieved rows from
    // the prepared statement
    ResultSet result_set = prepared_statement.executeQuery();

    System.out.println("Customer " + id +
      " details using JDBC connection:");

    // process the result set and display the customer details
    while (result_set.next()) {
      System.out.println("first name = " + result_set.getString(1));
      System.out.println("last name = " + result_set.getString(2));
    }

    // close the result set
    result_set.close();

    // close the prepared statement
    prepared_statement.close();

  } // end of displayCustomer()

}/*@lineinfo:generated-code*/class JdbcSqljExample1_SJProfileKeys 
{
  private static JdbcSqljExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new JdbcSqljExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private JdbcSqljExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = DefaultContext.getProfileKey(loader, "JdbcSqljExample1_SJProfile0");
  }
}
