/*@lineinfo:filename=PerformanceExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program PerformanceExample1.sqlj illustrates how to use row
   prefetching to improve performance of SQL queries.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;
import oracle.jdbc.driver.OracleConnection;

public class PerformanceExample1 {

  public static final int ROWS = 20000;
  public static final int PREFETCH_ROWS = 100;

  // declare a named iterator class
  /*@lineinfo:generated-code*//*@lineinfo:17^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

public static class PrefetchIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public PrefetchIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    valueNdx = findColumn("value");
  }
  public int value() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(valueNdx);
  }
  private int valueNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:19^3*/

  public static void main(String [] args) {

    try {

      // connect to a database on a remote machine
      Oracle.connect(
        "jdbc:oracle:thin:@remotehost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // create a temporary table
      /*@lineinfo:generated-code*//*@lineinfo:33^7*/

//  ************************************************************
//  #sql { CREATE TABLE prefetch_table (
//            value INTEGER
//          )
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:37^7*/

      // add rows to the table
      for (int count = 0; count < ROWS; count++) {
        /*@lineinfo:generated-code*//*@lineinfo:41^9*/

//  ************************************************************
//  #sql { INSERT INTO prefetch_table
//              (value)
//            VALUES
//              (:count)
//           };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = count;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:46^9*/
      }

      // display the current number of rows to prefetch
      int prefetch_rows = ((OracleConnection)
        DefaultContext.getDefaultContext().getConnection()).
          getDefaultRowPrefetch();
      System.out.println("Number of rows to be prefetched = " +
        prefetch_rows);

      retrieveRows();
      retrieveRows();
      retrieveRows();

      // set the number of rows to prefetch to PREFETCH_ROWS
      ((OracleConnection)
        DefaultContext.getDefaultContext().getConnection()).
          setDefaultRowPrefetch(PREFETCH_ROWS);

      // display the current number of rows to prefetch
      prefetch_rows = ((OracleConnection)
        DefaultContext.getDefaultContext().getConnection()).
          getDefaultRowPrefetch();
      System.out.println("Number of rows to be prefetched = " +
        prefetch_rows);

      retrieveRows();
      retrieveRows();
      retrieveRows();

      // drop prefetch_table
      /*@lineinfo:generated-code*//*@lineinfo:77^7*/

//  ************************************************************
//  #sql { DROP TABLE prefetch_table  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:77^40*/

      Oracle.close();

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void retrieveRows()
  throws SQLException {

    // create a SQLJ iterator
    PrefetchIteratorClass prefetch_iterator;

    long start_time = System.currentTimeMillis();

    // populate the iterator
    /*@lineinfo:generated-code*//*@lineinfo:100^5*/

//  ************************************************************
//  #sql prefetch_iterator = { SELECT
//          value
//        FROM
//          prefetch_table
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      prefetch_iterator = new PrefetchIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:105^5*/

    long end_time = System.currentTimeMillis();

    System.out.println("Total time for retrieving " + ROWS +
      " rows was " + (end_time - start_time) + " milliseconds.");

  } // end of retrieveRows()

}/*@lineinfo:generated-code*/class PerformanceExample1_SJProfileKeys 
{
  private static PerformanceExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new PerformanceExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private PerformanceExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "PerformanceExample1_SJProfile0");
  }
}
